/*
    Copyright 2011 Tianjun Liao Marco A. Montes de Oca, Dogan Aydin, Thomas Stuetzle and Marco Dorigo (IACOr-LS)
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle (IPSOLS)
    Code structure is developed by Marco A. Montes de Oca.

    This file is part of IACOr-LS.

    IACOr-LS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IACOr-LS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IACOr-LS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rng.h"
#include "configuration.h"
#include "swarm.h"
#include "problem.h"
#include "ACO.h"

ACO::ACO(Configuration* config, Swarm* swarm, Problem* problem){
	this->config = config;
	s = swarm;
	p = problem;	
}

ACO::~ACO(){}

double ACO::limitVelocity(double vel){
	if(vel > config->getMaxInitRange())
		return config->getMaxInitRange();
	else if(vel < config->getMinInitRange())
		return config->getMinInitRange();
	else
		return vel;
}

double ACO::clampPosition(double candidate){
	if(candidate > config->getMaxInitRange()) 
		return config->getMaxInitRange();
	else if(candidate < config->getMinInitRange()) 
		return config->getMinInitRange();
	else
		return candidate;
}

double ACO::clampPositionVelocity(double candidate,double* velocity){
	if(candidate > config->getMaxInitRange()) {
		*velocity = 0.0;
		return config->getMaxInitRange();
	}
	else if(candidate < config->getMinInitRange()) {
		*velocity = 0.0;
		return config->getMinInitRange();
	}
	else
		return candidate;
}

double ACO::reboundPosition(double candidate,double* velocity){
	double temp = candidate + *velocity;	
	if(temp > config->getMaxInitRange()) {		
		*velocity = -1*RNG::randVal(0.0,1.0)*(*velocity);
		if(*velocity > 0)					
			return config->getMaxInitRange() - *velocity;
		return config->getMaxInitRange() + *velocity;
	}
	else if(temp <  config->getMinInitRange()) {
		*velocity = -1*RNG::randVal(0.0,1.0)*(*velocity);
		if(*velocity < 0)					
			return  config->getMinInitRange() - *velocity;
		return  config->getMinInitRange() + *velocity;
	}
	else
		return temp;
}

//acor
double ACO::acorboundPosition(double candidate){
	double temp = candidate;
	if(temp > config->getMaxInitRange()) {
		return  config->getMaxInitRange()-RNG::randVal(0.0,1.0)*(temp-config->getMaxInitRange());
			}
	else if(temp <  config->getMinInitRange()) {
		return  config->getMinInitRange()+RNG::randVal(0.0,1.0)*(config->getMinInitRange()-temp);
	}
	else
		return temp;
}

void ACO::printSolution(){
	const double* solution = s->getSolution();
	for(unsigned int k=0;k<config->getProblemDimension();k++)
		printf("%lf ",solution[k]);
	printf("\n");
}

