/*
    Copyright 2011 Tianjun Liao Marco A. Montes de Oca, Dogan Aydin, Thomas Stuetzle and Marco Dorigo (IACOr-LS)
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle (IPSOLS)
    Code structure is developed by Marco A. Montes de Oca.

    This file is part of IACOr-LS.

    IACOr-LS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IACOr-LS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IACOr-LS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "configuration.h"
#include <tclap/CmdLine.h>


using namespace TCLAP;
using namespace std;

	Configuration::Configuration(int argc,char** argv){
		try {
			CmdLine cmd("Incremental PSO", ' ', "1.0");

			//Required seed value
			ValueArg<unsigned long> seedArg("","seed","RNG seed",true,0,"unsigned long");
			cmd.add(seedArg);

			//Required maximum function evaluations value
			ValueArg<unsigned int> fesArg("","fes","Maximum number of function evaluations",true,0,"unsigned int");
			cmd.add(fesArg);

			//Required problem ID
			vector<unsigned int> allowedProblems;

			allowedProblems.push_back( SHIFTEDGRIEWANK );
			allowedProblems.push_back( SHIFTEDACKLEY );

			ValuesConstraint<unsigned int> allowedProblemIDVals( allowedProblems );

 			ValueArg<unsigned int> pIDArg("","problemID","Problem ID",true,0,&allowedProblemIDVals);
			cmd.add(pIDArg);


			
			//Required dimension value
 			ValueArg<unsigned int> dimArg("","dimensions","Dimensionality",true,0,"unsigned int");
			cmd.add(dimArg);

			//Required minimum value of the search range
			ValueArg<double> minRangeArg("","minrange","Minimum value of the search range",true,0,"double");
			cmd.add(minRangeArg);

			//Required minimum value of the search range
			ValueArg<double> maxRangeArg("","maxrange","Maximum value of the search range",true,0,"double");
			cmd.add(maxRangeArg);

		
			//Max stagnant iterations
			ValueArg<unsigned int> sIArg("","stagnantiterations","Stagnant Iterations before a full restart",true,10,"unsigned int");
			cmd.add(sIArg);

			//Max repeated local search failures
			ValueArg<unsigned int> lsfArg("","lsfailures","Repeated local search failures",true,2,"unsigned int");
			cmd.add(lsfArg);


			//Number of evaluations for Local Search
 			ValueArg<unsigned int> lsItersArg("","lsIterations","LsIterations",true,0,"unsigned int");
			cmd.add(lsItersArg);

			ValueArg<double> ftolArg("","ftol","FTol",true,0.0,"double");
			cmd.add(ftolArg);

			
				//Required local search ID
			vector<unsigned int> allowedLocalSearchs;
			allowedLocalSearchs.push_back( NO_LOCAL_SEARCH );
			allowedLocalSearchs.push_back( POWELL );
			ValuesConstraint<unsigned int> allowedLocalSearchsIDVals( allowedLocalSearchs );

 			ValueArg<unsigned int> lsIDArg("","localSearch","Local Search",true,0,&allowedLocalSearchsIDVals);
			cmd.add(lsIDArg);


			//Required growth period
			ValueArg<unsigned int> growthArg("","growthperiod","Growth period",true,1,"unsigned int");
			cmd.add(growthArg);
			
			//Required maximum population size
			ValueArg<unsigned int> maxPopArg("","maxpopsize","Maximum population size",true,1000,"unsigned int");
			cmd.add(maxPopArg);
		
			//Required itial population size
			ValueArg<unsigned int> initPopArg("","initpopsize","Initial population size",true,1,"unsigned int");
			cmd.add(initPopArg);


			//acor
			// Required speed of convergence
			ValueArg<double> xiArg("","xi","speed of convergence",true,0,"double");
			cmd.add(xiArg);

			// select global or local method
			ValueArg<double> qArg("","q","selectmethods",true,0,"double");
			cmd.add(qArg);

			//Parses input
			cmd.parse( argc, argv );

			rngSeed = seedArg.getValue();
			maxFES = fesArg.getValue();

			problemID = pIDArg.getValue();

			localSearchID = lsIDArg.getValue();
			lsIters = lsItersArg.getValue();
			ftol = ftolArg.getValue();

			problemDimension = dimArg.getValue();
			minInitRange=minRangeArg.getValue();
			maxInitRange=maxRangeArg.getValue();


			initialPopulationSize = initPopArg.getValue();
			maxPopulationSize = maxPopArg.getValue();
			currentPopulationSize=0;
 			growthPeriod = growthArg.getValue();
			maxStagnantIterations = sIArg.getValue();
			maxRepeatedFailures = lsfArg.getValue();


			//acor
			Xi=xiArg.getValue();
			q=qArg.getValue();

		}catch (TCLAP::ArgException &e)  // catch any exceptions
		{ std::cerr << "error: " << e.error() << " for arg " << e.argId() << std::endl; }
	}

	Configuration::~Configuration(){}

	unsigned long Configuration::getRNGSeed(){
		return rngSeed;
	}
 	unsigned int Configuration::getMaxFES(){
		return maxFES;
	}
	unsigned int Configuration::getProblemID(){
		return problemID;
	}

	unsigned int Configuration::getLocalSearchID(){
		return localSearchID;
	}

	unsigned int Configuration::getLsIterations(){
		return lsIters;
	}

	unsigned int Configuration::getProblemDimension(){
		return problemDimension;
	}
	double Configuration::getMinInitRange(){
		return minInitRange;
	}
      	double Configuration::getMaxInitRange(){
		return maxInitRange;
	}

	double Configuration::getFTol(){
		return pow(10.0,ftol);
	}

	unsigned int Configuration::getInitialPopulationSize(){
		return initialPopulationSize;
	}

	unsigned int Configuration::getMaximumPopulationSize(){
		return maxPopulationSize;
	}

	unsigned int Configuration::getGrowthPeriod(){
		return growthPeriod;
	}

	unsigned int Configuration::getMaxStagnantIterations(){
		return maxStagnantIterations;
	}

	unsigned int Configuration::getMaxRepeatedFailures(){
		return maxRepeatedFailures;
	}

	void Configuration::setStartTime(double stime){
		startTime = stime;
	}

	double Configuration::getStartTime(){
		return startTime;
	}

	void Configuration::setCurrentPopulationSize(unsigned int newSize){
		currentPopulationSize = newSize;
	}

	unsigned int  Configuration::getCurrentPopulationSize(){
		return currentPopulationSize;
	}

	double Configuration::getxi(){
			return Xi;
		}
		double Configuration::getq(){
			return q;
		}

	void Configuration::print(){
		cout << "Parameters:" << endl;
		cout << endl;
		cout << "rngSeed = " << rngSeed << endl;
		cout <<	"maxFES = " << maxFES << endl;
		cout << "problemID = " << problemID << endl;
		cout << "localSearch = " << localSearchID << endl;
		cout << "FTol = " << ftol << endl;

		cout << "problemDimension = " << problemDimension << endl;
		cout << "minInitRange = " << minInitRange << endl;
		cout << "maxInitRange = " << maxInitRange << endl;
		cout << "maxPopulationSize = " << maxPopulationSize << endl;
		//acor
		cout << "xi = " << Xi << endl;
		cout <<	"q = " <<q << endl;
	}


