/*
    Copyright 2011 Tianjun Liao Marco A. Montes de Oca, Dogan Aydin, Thomas Stuetzle and Marco Dorigo (IACOr-LS)
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle (IPSOLS)
    Code structure is developed by Marco A. Montes de Oca.

    This file is part of IACOr-LS.

    IACOr-LS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IACOr-LS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IACOr-LS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "localSearch.h"
#include "localSearchProblems.h"
#include "particle.h"
#include "rng.h"
#include "powell.h"

#include <iostream>
#include <cfloat>
using namespace std;


//for mtsls1;
bool improve;

int grade;
int bonus1=10;
int bonus2=1;


LocalSearch::LocalSearch(Problem* problem, Configuration* config)
{
    this->config = config;
    this->problem = problem;
}

LocalSearch::~LocalSearch(){}


void LocalSearch::apply(Particle * part, double stepSize){

    const double* pos = part->getBestPosition();
    double x[config->getProblemDimension()];

    // create a temporary position vector for the particle position
    for(int i = 0; i < config->getProblemDimension(); i++)
            x[i] = pos[i];

    long double min_val=LDBL_MAX;  /* the minimum objective value, upon return */
    
    // required data for controling and printing progress inside of local search method  
    long double current[7] ={(long double)problem->getFunctionEvaluations(),problem->getBestSolutionValue(), EPSILON, config->getStartTime(),config->getCurrentPopulationSize(), config->getMinInitRange(),config->getMaxInitRange()};

    // get objective function
    obj_func func = getProblemFunc(config->getProblemID());

    // apply local search, returns the termination code, the result is stored in min_val and current[1]
    int ret = powell(config->getProblemDimension(), x, stepSize, config->getFTol(), config->getLsIterations(), func, current, &min_val);
    
    if(current[1] < EPSILON)
	current[1] = 0.0;

    // update current number of function evaluations
    problem->setFunctionEvaluations((unsigned int)current[0]);
    
    //update best solution value
    if(current[1] < problem->getBestSolutionValue()){
            problem->setBestSolutionValue(current[1]);
	    problem->setBestSoFarSolution(x);
    }

    //update best position and value of the particle
    if(part->getBestValue() > current[1]){
            part->setBestValue(current[1]);
            part->setBestPosition(x);
    }

    switch(ret){
	case FTOL_REACHED: { 
		part->setToImprove(false);
	}break;
       	case MAXEVAL_REACHED: {
		part->setToImprove(true);
	}break;
	default:{
		cout << "Error" << endl;
		exit(1);
	}
    }

}



obj_func LocalSearch::getProblemFunc(int funcID)
{

    obj_func func;

    switch (funcID){


            case SHIFTEDGRIEWANK:
                func = shiftedGriewankFunction;
                break;
            case SHIFTEDACKLEY:
                func = shiftedAckleyFunction;
                break;


            default:
                func = NULL;
                cout<<"Error: Unexpected function ID"<<endl;
        }

        return func;

}

void LocalSearch::applymtsls1(Particle * part, double SR)
{
	const double* pos = part->getBestPosition();
	    double x[config->getProblemDimension()];

	    // create a temporary position vector for the particle position
	    for(int i = 0; i < config->getProblemDimension(); i++)
	            x[i] = pos[i];

	    //long double min_val=LDBL_MAX;  /* the minimum objective value, upon return */

	    // required data for controling and printing progress inside of local search method
		 long double current[7] ={(long double)problem->getFunctionEvaluations(),problem->getBestSolutionValue(), EPSILON, config->getStartTime(),config->getCurrentPopulationSize(), config->getMinInitRange(),config->getMaxInitRange()};

		 //get objective function
		 obj_func func = getProblemFunc(config->getProblemID());

		// improve = TRUE
         improve =true;

         grade=0;

         //double SR=(config->getMaxInitRange()-config->getMinInitRange())/(long double)(2);
		// apply local search, returns the termination code, the result is stored in min_val and current[1]

         int ret = mtsls1(x, SR, part,func ,current);

		if(current[1] < EPSILON)
		current[1] = 0.0;

		// update current number of function evaluations
		problem->setFunctionEvaluations((unsigned int)current[0]);

		//update best solution value
		if(current[1] < problem->getBestSolutionValue()){
				problem->setBestSolutionValue(current[1]);
			problem->setBestSoFarSolution(x);
		}

		//update best position and value of the particle
		if(part->getBestValue() > current[1]){
				part->setBestValue(current[1]);
				part->setBestPosition(x);
		}


		if(ret==0)
		{
			part->setToImprove(false);
		}
		else if(ret>0)
		{
			part->setToImprove(true);
		}
		else
		{
			cout << "Error" << endl;
						exit(1);
		}
/*
		switch(ret){
		case : {
			part->setToImprove(false);
		}break;
			case MAXEVAL_REACHED: {
			part->setToImprove(true);
		}break;
		default:{
			cout << "Error" << endl;
			exit(1);
		}
		}
*/
}
int LocalSearch::mtsls1(

	double xk[],
	double s,
	Particle * partlocal,
	obj_func fun,
    long double * fun_data )
{
int iter=0;
vector<int> conbef;
bool convergence = false;
do
{

   if(improve == false)
   {

      s=s/(long double)(2);
     // the value somehow is important ex: f15
      if(s<1e-20)
      {

         conbef.push_back(problem->getBestSolutionValue());
         if(conbef.size()>=2)
         {
        	 if(abs(conbef.back()-conbef[conbef.size()-1])<1e-20)
         	 {
        		 //cout<<"############"<<endl;
        		 convergence=true;

			 }


         }

    	s=RNG::randVal(0.3,0.6)*(config->getMaxInitRange()-config->getMinInitRange());
    	 //s=0.4*(config->getMaxInitRange()-config->getMinInitRange());
      }
   }

 // need to judge converge

  // cout<<"SR is "<<s<<endl;
/*
      for(int i = 0; i < config->getProblemDimension(); i++)
      {
          cout<<xk[i]<<" ";
      }
      cout<<fun(config->getProblemDimension(), xk, fun_data);
      cout<<endl;
*/

   improve=false;
   for(int i = 0; i < config->getProblemDimension(); i++)
   {
	   long double before1=fun(config->getProblemDimension(), xk, fun_data);
	   xk[i]=xk[i]-s;


	   long double after1=fun(config->getProblemDimension(), xk, fun_data);
	   // compared to the current best solution

	   if(after1<problem->getBestSolutionValue())
	   {
		   grade=grade+bonus1;

		   partlocal->setBestPosition(xk);
		   problem->setBestSolutionValue(after1);

	   }

	   if(abs(after1-before1)<=1e-20)
	   {
		   xk[i]=xk[i]+s;
	   }
	   else
	   {
		   if(after1-before1>1e-20)
		   {
			   xk[i]=xk[i]+s;
			   long double before2=before1;
			   xk[i]=xk[i]+0.5*s;

			   long double after2=fun(config->getProblemDimension(), xk, fun_data);

			   if(abs(after2-problem->getBestSolutionValue())>1e-20&&after2<problem->getBestSolutionValue())
			   {
				   grade=grade+bonus1;
				   //partlocal->setCurrentPosition(xk);
				   //partlocal->evaluateCurrentPosition();
				  partlocal->setBestPosition(xk);
				   problem->setBestSolutionValue(after2);
			   }

			   if(after2>=before2||(0<before2-after2<=1e-20))
			   {
				   xk[i]=xk[i]-0.5*s;
			   }
			   else
			   {
				   grade=grade+bonus2;
				   improve=true;
			   }


		   }
		   else
		   {
			   grade=grade+bonus2;
			   improve=true;
		   }

	   }
   }
   iter++;
}while(iter<config->getLsIterations()&& convergence == false);

//cout<<"i am out i am out"<<endl;
return grade;


}
