/*
    Copyright 2011 Tianjun Liao Marco A. Montes de Oca, Dogan Aydin, Thomas Stuetzle and Marco Dorigo (IACOr-LS)
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle (IPSOLS)
    Code structure is developed by Marco A. Montes de Oca.

    This file is part of IACOr-LS.

    IACOr-LS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IACOr-LS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IACOr-LS.  If not, see <http://www.gnu.org/licenses/>.
*/


//Configuration
#include "configuration.h"

//Problems
#include "problem.h"


#include "shiftedgriewank.h" 		//f5
#include "shiftedackley.h" 		//f6


//Algorithms
#include "ACO.h"
#include "incrementalACO.h"

//Timers
#include <sys/time.h>
#include <sys/resource.h>

//C++ Libraries
#include <iostream>


using namespace std;


Problem* initializeProblem(Configuration * config){

    Problem* problem;

    //Problem initialization
	switch(config->getProblemID()){

		case SHIFTEDGRIEWANK:{
			problem = new ShiftedGriewank(config);
		}break;
		case SHIFTEDACKLEY:{
			problem = new ShiftedAckley(config);
		}break;


	}

	return problem;

}

int main( int argc, char** argv) {	

	double stime;
	static struct timeval tp;	

	Problem* problem;

	//Command line parsing
	Configuration* config = new Configuration(argc,argv);

	//Random number generator
	RNG::initializeRNG(config->getRNGSeed());

    	//Initialize Problem
	problem = initializeProblem(config);

	Swarm* swarm = new Swarm(config,problem);

	//Start timer
	gettimeofday( &tp, NULL );
   	stime =(double) tp.tv_sec + (double) tp.tv_usec/1000000.0;
	config->setStartTime(stime);
	
	//ACO intialization
	ACO* ACO = new IncrementalACO(config,swarm,problem);
	
	//ACO execution
	ACO->run();

	//Memory release
	RNG::deallocateRNG();
	RNG::deallocatePermutation();
	delete swarm;
	delete problem;
	delete ACO;
	delete config;

	return 0;
}
