/*
    Copyright 2011 Tianjun Liao Marco A. Montes de Oca, Dogan Aydin, Thomas Stuetzle and Marco Dorigo (IACOr-LS)
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle (IPSOLS)
    Code structure is developed by Marco A. Montes de Oca.

    This file is part of IACOr-LS.

    IACOr-LS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IACOr-LS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IACOr-LS.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PROBLEM_H
#define PROBLEM_H

#include "configuration.h"
#include <cmath>
#include <algorithm>

#define abss(a)     (a<0 ? (-a) : a)


class Problem {

      protected:
	Configuration* config;
	int dimension;
	unsigned int evaluations;
	unsigned int maxEvaluations;
	double optimum;
	long double oldResult;

	bool firstEvaluation;
	long double bestSolutionValue;
	double* bestSoFarSolution;


	static const double griewank[];
	static const double ackley[];

    

      public:

	 Problem(Configuration* config);
	 virtual ~Problem();

	int getProblemDimension();
	double getProblemOptimum();

	unsigned int getFunctionEvaluations();

	void setFunctionEvaluations(unsigned int evals);

	//Evaluation call
	virtual long double getFunctionValue(const double* x) = 0;

	long double getBestSolutionValue();
	double* getBestSoFarSolution();
	void setBestSoFarSolution(const double* solution);

        void setBestSolutionValue(long double value);

	void printProgress();

	void divideFunctions(const double *s,  double *part1, double *part2, double m, int *psize1, int *psize2);

};


#endif

