/*
    Copyright 2011 Tianjun Liao Marco A. Montes de Oca, Dogan Aydin, Thomas Stuetzle and Marco Dorigo (IACOr-LS)
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle (IPSOLS)
    Code structure is developed by Marco A. Montes de Oca.

    This file is part of IACOr-LS.

    IACOr-LS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IACOr-LS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IACOr-LS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rng.h"
#include "problem.h"
#include "shiftedackley.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

ShiftedAckley::ShiftedAckley(Configuration* config):Problem(config){
	optimum = 0.0;
}

ShiftedAckley::~ShiftedAckley(){
}

long double ShiftedAckley::getFunctionValue(const double* x) {
	long double result;

	if(evaluations < maxEvaluations){

		result = shiftedAckleyFunction(x);

		//Update number of function evaluations
		evaluations++;

		oldResult  = result;
		
		
		//Update and print best solution value
		if( bestSolutionValue > result){
			bestSolutionValue = result;
			setBestSoFarSolution(x);
			printProgress();
		}
		
		return result;
	}
	else
		return oldResult;

}

long double ShiftedAckley::shiftedAckleyFunction(const double* x){
    int i;
    long double z;
    long double Sum1 = 0;
    long double Sum2 = 0;

    for(i=0;i<dimension;i++){
        z = x[i] - ackley[i];
        Sum1 = Sum1 + pow(z , 2 );
        Sum2 = Sum2 + cos(2*PI*z);
    }
    
    return -20*exp(-0.2*sqrt(Sum1/(double)dimension)) -exp(Sum2/(double)dimension) + 20 + exp(1);
}
