/*
    Copyright 2011 Tianjun Liao Marco A. Montes de Oca, Dogan Aydin, Thomas Stuetzle and Marco Dorigo (IACOr-LS)
    Copyright 2010 Marco A. Montes de Oca, Dogan Aydin and Thomas Stuetzle (IPSOLS)
    Code structure is developed by Marco A. Montes de Oca.

    This file is part of IACOr-LS.

    IACOr-LS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    IACOr-LS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with IACOr-LS.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "rng.h"
#include "problem.h"
#include "shiftedgriewank.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

ShiftedGriewank::ShiftedGriewank(Configuration* config):Problem(config){
	optimum = 0.0;
}

ShiftedGriewank::~ShiftedGriewank(){
}

long double ShiftedGriewank::getFunctionValue(const double* x) {
	long double result;

	if(evaluations < maxEvaluations){

		result = shiftedGriewankFunction(dimension, x);

		//Update number of function evaluations
		evaluations++;

		oldResult  = result;
		
		
		//Update and print best solution value
		if( bestSolutionValue > result){
			bestSolutionValue = result;
			setBestSoFarSolution(x);
			printProgress();
		}		

		return result;
	}
	else
		return oldResult;

}

long double ShiftedGriewank::shiftedGriewankFunction(int dim, const double* x){
    int i;
    double z;
    long double F1 = 0.0;
    long double F2 = 1.0;

    for(i=0;i<dim;i++){
        z = x[i] - griewank[i];
        F1 = F1 + ( pow(z,2.0) / 4000.0 );
        F2 = F2 * ( cos(z/sqrt(i+1)));

    }

    return F1 - F2 + 1.0;
}
